"use strict";

const Promise = require("bluebird");
const memoizee = require("memoizee");
const fs = Promise.promisifyAll(require("fs"));
const findmnt = require("../../packages/exec-findmnt");
const All = require("../../packages/graphql-interface/symbols/all");
const treeMapAsync = require("../../packages/tree-map-async");

module.exports = function () {
	let findmntOnce = memoizee(() => {
		return Promise.try(() => {
			return findmnt();
		}).then((mounts) => {
			return treeMapAsync(mounts, async (mount) => {
				if (mount.sourceDevice?.startsWith("/")) {
					return {
						... mount,
						sourceDevice: await fs.realpathAsync(mount.sourceDevice)
					};
				} else {
					// Skip mounts that don't exist at a path at all
					return mount;
				}
			}, true);
		});
	});

	return function (mountpoints) {
		return Promise.try(() => {
			return findmntOnce();
		}).then(({tree, list}) => {
			return mountpoints.map((mountpoint) => {
				if (mountpoint === All) {
					return tree;
				} else {
					return list.find((mount) => mount.mountpoint === mountpoint);
				}
			});
		});
	};
};
